/*
 * QueryExecutionOutput class.
 *
 * Copyright (C) 2012 SATOH Takayuki All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
package ts.query;

import ts.util.AbstractTypedGetter;
import java.io.Serializable;
import java.util.Map;
import java.util.HashMap;

/**
 * クエリ出力項目クラス。
 * <br>
 * クエリの実行結果として取得できる出力項目を表すクラス。
 * 出力項目の名前を属性に持ち、その他にデータ型や桁数などの任意の属性を保持する
 * ことができる。
 *
 * @author 佐藤隆之
 * @version $Id: QueryExecutionOutput.java,v 1.4 2012-03-09 16:12:34 tayu Exp $
 */
public class QueryExecutionOutput
  extends AbstractTypedGetter<Enum<?>,Serializable>
{
  /** シリアル・バージョン番号。 */
  static final long serialVersionUID = 600184640731187305L;

  /** 出力項目名。 */
  private final String name;

  /** 任意の属性を格納するマップ。 */
  private Map<Enum<?>,Serializable> attributeMap = null;

  /**
   * 出力項目名を引数にとるコンストラクタ。
   *
   * @param name 出力項目名。 
   * @throws AssertionError 引数がヌルの場合（デバッグ・モードのみ）。
   */
  public QueryExecutionOutput(String name)
  {
    assert (name != null) : "@param:name is null.";
    this.name = name;
  }

  /**
   * 出力項目名を取得する。
   *
   * @return 出力項目名。
   */
  public String getName()
  {
    return this.name;
  }

  /**
   * このオブジェクトのハッシュ・コードを取得する。
   *
   * @return このオブジェクトのハッシュ・コード。
   */
  @Override
  public int hashCode()
  {
    return this.name.hashCode();
  }

  /**
   * 引数のオブジェクトがこのオブジェクトと等しいかどうかを判定する。
   * <br>
   * 引数のオブジェクトが{@link QueryExecutionOutput}オブジェクトで、且つ
   * 出力項目名が一致するならば、等しいと判定して<tt>true</tt>を返す。
   *
   * @return 引数のオブジェクトがこのオブジェクトと等しい場合は<tt>true</tt>。
   */
  @Override
  public boolean equals(Object obj)
  {
    if (obj == null) {
      return false;
    }

    if (! (obj instanceof QueryExecutionOutput)) {
      return false;
    }

    QueryExecutionOutput output = QueryExecutionOutput.class.cast(obj);
    if (! this.name.equals(output.name)) {
      return false;
    }

    return true;
  }

  /**
   * 指定されたキーに結びつけられた属性値を取得する。
   *
   * @param attrKey キー。
   * @return 属性値。
   */
  @Override
  public Serializable get(Enum<?> attrKey)
  {
    return (this.attributeMap == null) ? null : this.attributeMap.get(attrKey);
  }

  /**
   * 指定されたキーを属性名とする属性値を設定する。
   *
   * @param attrKey キー。
   * @param attrValue 属性値。
   * @return 以前にこのキーに結びつけられていた属性値。
   */
  public Serializable put(Enum<?> attrKey, Serializable attrValue)
  {
    if (this.attributeMap == null) {
      this.attributeMap = newAttributeMap();
    }

    if (attrValue == null) {
      return this.attributeMap.remove(attrKey);
    }
    else {
      return this.attributeMap.put(attrKey, attrValue);
    }
  }

  /**
   * 属性のキーと値の関連付けを格納するマップを作成する。
   *
   * @return 属性マップ。
   */
  protected Map<Enum<?>,Serializable> newAttributeMap()
  {
    return new HashMap<Enum<?>,Serializable>();
  }
}
